<?php

namespace ModulesGarden\WithoutDNS\Share\Helpers;

require_once __DIR__ . DIRECTORY_SEPARATOR . "Database.php";

class DomainsUrls
{
    public const TRIAL = 'trial';
    public const MAIN = 'main';

    /**
     * Get all records from the domain_url table.
     *
     * @return array
     */
    public static function getAllRecords(): array
    {
        $database = new Database();

        $query = "SELECT * FROM domain_url;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->execute();

        $rows = $stmt->fetchAll($database->getConnection()::FETCH_ASSOC);

        // Process each row to unserialize and decode the 'data' column
        $processedRows = [];
        foreach ($rows as $row) {
            if (isset($row['data'])) {
                // Step 1: Unserialize the data
                $unserializedData = unserialize($row['data']);

                // Step 2: Decode the JSON string
                if ($unserializedData !== false) {
                    $decodedData = json_decode($unserializedData, true);

                    if ($decodedData === null && json_last_error() !== JSON_ERROR_NONE) {
                        $decodedData = null;
                    }
                } else {
                    $decodedData = null;
                }

                $row['data'] = $decodedData;
            }

            // Add the processed row to the result
            $processedRows[] = $row;
        }

        return $processedRows;
    }

    /**
     * Get a record by domain_name.
     *
     * @param string $domainName
     * @return array|null
     */
    public static function getByDomainName(string $domainName): ?array
    {
        $database = new Database();
        $query = "SELECT * FROM domain_url WHERE domain_name = :domain_name";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(':domain_name', $domainName);
        $stmt->execute();

        $row = $stmt->fetch($database->getConnection()::FETCH_ASSOC);

        if ($row && isset($row['data'])) {
            // Unserialize and decode JSON
            $unserializedData = unserialize($row['data']);
            $decodedData = $unserializedData !== false ? json_decode($unserializedData, true) : null;

            if ($decodedData === null && json_last_error() !== JSON_ERROR_NONE) {
                $decodedData = null;
            }

            $row['data'] = $decodedData;
        }

        return $row ?: null;
    }

    /**
     * Delete a record by domain_name.
     *
     * @param string $domainName
     * @return bool
     */
    public static function deleteByDomainName(string $domainName): bool
    {
        $database = new Database();
        $query = "DELETE FROM domain_url WHERE domain_name = :domain_name";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(':domain_name', $domainName);
        $stmt->execute();

        return $stmt->rowCount() > 0;
    }

    /**
     * Get a single record by domain_name or return a default value if not found.
     *
     * @param string $domainName
     * @param mixed $default
     * @return mixed
     */
    public static function getByDomainNameOrDefault(string $domainName, $default = null)
    {
        $database = new Database();

        $query = "SELECT * FROM domain_url WHERE domain_name = :domain_name;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(":domain_name", $domainName);
        $stmt->execute();

        $result = $stmt->fetch($database->getConnection()::FETCH_ASSOC);

        // Return the record if found, otherwise return the default value
        return $result ? $result : $default;
    }

    /**
     * Update or create a record based on domain_name.
     *
     * @param string $domainName
     * @param string $data
     * @return void
     */
    public static function updateOrCreate(string $domainName, string $data): void
    {
        $database = new Database();

        // Check if the record already exists
        $query = "SELECT * FROM domain_url WHERE domain_name = :domain_name;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(":domain_name", $domainName);
        $stmt->execute();

        $existingRecord = $stmt->fetch($database->getConnection()::FETCH_ASSOC);

        if ($existingRecord) {
            // Update the existing record
            $query = "UPDATE domain_url SET data = :data WHERE domain_name = :domain_name;";
        } else {
            // Insert a new record
            $query = "INSERT INTO domain_url (domain_name, data) VALUES (:domain_name, :data);";
        }

        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(":domain_name", $domainName);
        $stmt->bindParam(":data", $data);
        $stmt->execute();
    }

    /**
     * Find a record in the provided records based on a specific condition.
     *
     * @param array $records The records to search through.
     * @param string $column The column to search in.
     * @param mixed $value The value to match.
     * @return array|null The matching record or null if not found.
     */
    public static function findRecordInAllRecords(array $records, string $column, mixed $value): ?array
    {
        // Loop through records to find a match
        foreach ($records as $record) {
            if (isset($record[$column]) && $record[$column] === $value) {
                return $record;
            }
        }

        // Return null if no match is found
        return null;
    }

    /**
     * Delete records from the domain_url table where:
     * - valid_until is not "0000-00-00 00:00:00" and is earlier than the current date/time.
     * - valid_until does not exist in the data column.
     *
     * @return int Number of deleted records.
     */
    public static function deleteExpiredRecords(): int
    {
        $database = new Database();

        // Get all records from the domain_url table
        $query = "SELECT id, data FROM domain_url;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->execute();

        $rows = $stmt->fetchAll($database->getConnection()::FETCH_ASSOC);

        // Track the number of deleted records
        $deletedCount = 0;

        // Process each row
        foreach ($rows as $row) {
            if (isset($row['data'])) {
                // Step 1: Unserialize the data
                $unserializedData = unserialize($row['data']);

                // Step 2: Decode the JSON string
                if ($unserializedData !== false) {
                    $decodedData = json_decode($unserializedData, true);

                    if ($decodedData === null && json_last_error() !== JSON_ERROR_NONE) {
                        // If JSON decoding fails, skip this row
                        continue;
                    }

                    // Check if 'valid_until' exists and is not "0000-00-00 00:00:00"
                    if (isset($decodedData['valid_until'])) {
                        $validUntil = $decodedData['valid_until'];

                        if ($validUntil !== "0000-00-00 00:00:00") {
                            // Convert 'valid_until' to a timestamp and compare with the current time
                            $validUntilTimestamp = strtotime($validUntil);
                            $currentTimestamp = time();

                            if ($validUntilTimestamp < $currentTimestamp) {
                                // Delete the record
                                $deleteQuery = "DELETE FROM domain_url WHERE id = :id;";
                                $deleteStmt = $database->getConnection()->prepare($deleteQuery);
                                $deleteStmt->bindValue(':id', $row['id']);
                                $deleteStmt->execute();

                                $deletedCount++;
                            }
                        }
                    } else {
                        // If 'valid_until' does not exist, delete the record
                        $deleteQuery = "DELETE FROM domain_url WHERE id = :id;";
                        $deleteStmt = $database->getConnection()->prepare($deleteQuery);
                        $deleteStmt->bindValue(':id', $row['id']);
                        $deleteStmt->execute();

                        $deletedCount++;
                    }
                }
            }
        }

        return $deletedCount;
    }
}